@@ -21,13 +21,13 @@ import netaddr
 
 from ryu.ofproto import ether
 from ryu.ofproto import inet
-from ryu.ofproto import ofproto_v1_2
-from ryu.ofproto import ofproto_v1_2_parser
+from ryu.ofproto import ofproto_v1_3
+from ryu.ofproto import ofproto_v1_3_parser
 from ryu.lib import hub
 from ryu.lib import mac
 
 
-LOG = logging.getLogger('ryu.lib.ofctl_v1_2')
+LOG = logging.getLogger('ryu.lib.ofctl_v1_3')
 
 DEFAULT_TIMEOUT = 1.0
 
@@ -88,6 +88,11 @@ def to_action(dp, dic):
         field = dic.get('field')
         value = dic.get('value')
         result = parser.OFPActionSetField(**{field: value})
+    elif action_type == 'PUSH_PBB':
+        ethertype = int(dic.get('ethertype'))
+        result = parser.OFPActionPushPbb(ethertype)
+    elif action_type == 'POP_PBB':
+        result = parser.OFPActionPopPbb()
     else:
         LOG.debug('Unknown action type: %s' % action_type)
 
@@ -117,6 +122,9 @@ def to_actions(dp, acts):
                 inst.append(
                     parser.OFPInstructionWriteMetadata(
                         metadata, metadata_mask))
+            elif action_type == 'METER':
+                meter_id = int(a.get('meter_id'))
+                inst.append(parser.OFPInstructionMeter(meter_id))
             else:
                 LOG.debug('Unknown action type: %s' % action_type)
 
@@ -128,34 +136,38 @@ def to_actions(dp, acts):
 def action_to_str(act):
     action_type = act.cls_action_type
 
-    if action_type == ofproto_v1_2.OFPAT_OUTPUT:
+    if action_type == ofproto_v1_3.OFPAT_OUTPUT:
         buf = 'OUTPUT:' + str(act.port)
-    elif action_type == ofproto_v1_2.OFPAT_COPY_TTL_OUT:
+    elif action_type == ofproto_v1_3.OFPAT_COPY_TTL_OUT:
         buf = 'COPY_TTL_OUT'
-    elif action_type == ofproto_v1_2.OFPAT_COPY_TTL_IN:
+    elif action_type == ofproto_v1_3.OFPAT_COPY_TTL_IN:
         buf = 'COPY_TTL_IN'
-    elif action_type == ofproto_v1_2.OFPAT_SET_MPLS_TTL:
+    elif action_type == ofproto_v1_3.OFPAT_SET_MPLS_TTL:
         buf = 'SET_MPLS_TTL:' + str(act.mpls_ttl)
-    elif action_type == ofproto_v1_2.OFPAT_DEC_MPLS_TTL:
+    elif action_type == ofproto_v1_3.OFPAT_DEC_MPLS_TTL:
         buf = 'DEC_MPLS_TTL'
-    elif action_type == ofproto_v1_2.OFPAT_PUSH_VLAN:
+    elif action_type == ofproto_v1_3.OFPAT_PUSH_VLAN:
         buf = 'PUSH_VLAN:' + str(act.ethertype)
-    elif action_type == ofproto_v1_2.OFPAT_POP_VLAN:
+    elif action_type == ofproto_v1_3.OFPAT_POP_VLAN:
         buf = 'POP_VLAN'
-    elif action_type == ofproto_v1_2.OFPAT_PUSH_MPLS:
+    elif action_type == ofproto_v1_3.OFPAT_PUSH_MPLS:
         buf = 'PUSH_MPLS:' + str(act.ethertype)
-    elif action_type == ofproto_v1_2.OFPAT_POP_MPLS:
+    elif action_type == ofproto_v1_3.OFPAT_POP_MPLS:
         buf = 'POP_MPLS'
-    elif action_type == ofproto_v1_2.OFPAT_SET_QUEUE:
+    elif action_type == ofproto_v1_3.OFPAT_SET_QUEUE:
         buf = 'SET_QUEUE:' + str(act.queue_id)
-    elif action_type == ofproto_v1_2.OFPAT_GROUP:
+    elif action_type == ofproto_v1_3.OFPAT_GROUP:
         buf = 'GROUP:' + str(act.group_id)
-    elif action_type == ofproto_v1_2.OFPAT_SET_NW_TTL:
+    elif action_type == ofproto_v1_3.OFPAT_SET_NW_TTL:
         buf = 'SET_NW_TTL:' + str(act.nw_ttl)
-    elif action_type == ofproto_v1_2.OFPAT_DEC_NW_TTL:
+    elif action_type == ofproto_v1_3.OFPAT_DEC_NW_TTL:
         buf = 'DEC_NW_TTL'
-    elif action_type == ofproto_v1_2.OFPAT_SET_FIELD:
+    elif action_type == ofproto_v1_3.OFPAT_SET_FIELD:
         buf = 'SET_FIELD: {%s:%s}' % (act.key, act.value)
+    elif action_type == ofproto_v1_3.OFPAT_PUSH_PBB:
+        buf = 'PUSH_PBB:' + str(act.ethertype)
+    elif action_type == ofproto_v1_3.OFPAT_POP_PBB:
+        buf = 'POP_PBB'
     else:
         buf = 'UNKNOWN'
     return buf
@@ -166,23 +178,28 @@ def actions_to_str(instructions):
 
     for instruction in instructions:
         if isinstance(instruction,
-                      ofproto_v1_2_parser.OFPInstructionActions):
+                      ofproto_v1_3_parser.OFPInstructionActions):
             for a in instruction.actions:
                 actions.append(action_to_str(a))
 
         elif isinstance(instruction,
-                        ofproto_v1_2_parser.OFPInstructionGotoTable):
+                        ofproto_v1_3_parser.OFPInstructionGotoTable):
             buf = 'GOTO_TABLE:' + str(instruction.table_id)
             actions.append(buf)
 
         elif isinstance(instruction,
-                        ofproto_v1_2_parser.OFPInstructionWriteMetadata):
+                        ofproto_v1_3_parser.OFPInstructionWriteMetadata):
             buf = ('WRITE_METADATA:0x%x/0x%x' % (instruction.metadata,
                                                  instruction.metadata_mask)
                    if instruction.metadata_mask
                    else 'WRITE_METADATA:0x%x' % instruction.metadata)
             actions.append(buf)
 
+        elif isinstance(instruction,
+                        ofproto_v1_3_parser.OFPInstructionMeter):
+            buf = 'METER:' + str(instruction.meter_id)
+            actions.append(buf)
+
         else:
             continue
 
@@ -191,7 +208,6 @@ def actions_to_str(instructions):
 
 def to_match(dp, attrs):
     match = dp.ofproto_parser.OFPMatch()
-
     convert = {'in_port': int,
                'in_phy_port': int,
                'dl_src': to_match_eth,
@@ -206,6 +222,8 @@ def to_match(dp, attrs):
                'nw_proto': int,
                'tp_src': int,
                'tp_dst': int,
+               'mpls_label': int,
+               'metadata': to_match_metadata,
                'eth_src': to_match_eth,
                'eth_dst': to_match_eth,
                'eth_type': int,
@@ -234,10 +252,13 @@ def to_match(dp, attrs):
                'ipv6_nd_target': to_match_ipv6,
                'ipv6_nd_sll': mac.haddr_to_bin,
                'ipv6_nd_tll': mac.haddr_to_bin,
-               'mpls_tc': int}
+               'mpls_tc': int,
+               'mpls_bos': int,
+               'pbb_isid': int,
+               'tunnel_id': int,
+               'ipv6_exthdr': int}
 
     match_append = {'in_port': match.set_in_port,
-                    'in_phy_port': match.set_in_phy_port,
                     'dl_src': match.set_dl_src_masked,
                     'dl_dst': match.set_dl_dst_masked,
                     'dl_type': match.set_dl_type,
@@ -248,6 +269,8 @@ def to_match(dp, attrs):
                     'nw_proto': match.set_ip_proto,
                     'tp_src': to_match_tpsrc,
                     'tp_dst': to_match_tpdst,
+                    'mpls_label': match.set_mpls_label,
+                    'metadata': match.set_metadata_masked,
                     'eth_src': match.set_dl_src_masked,
                     'eth_dst': match.set_dl_dst_masked,
                     'eth_type': match.set_dl_type,
@@ -278,7 +301,11 @@ def to_match(dp, attrs):
                     'ipv6_nd_target': match.set_ipv6_nd_target,
                     'ipv6_nd_sll': match.set_ipv6_nd_sll,
                     'ipv6_nd_tll': match.set_ipv6_nd_tll,
-                    'mpls_tc': match.set_mpls_tc}
+                    'mpls_tc': match.set_mpls_tc,
+                    'mpls_bos': match.set_mpls_bos,
+                    'pbb_isid': match.set_pbb_isid,
+                    'tunnel_id': match.set_tunnel_id,
+                    'ipv6_exthdr': match.set_ipv6_exthdr}
 
     if attrs.get('dl_type') == ether.ETH_TYPE_ARP or \
             attrs.get('eth_type') == ether.ETH_TYPE_ARP:
@@ -313,6 +340,11 @@ def to_match(dp, attrs):
                     key == 'udp_src' or key == 'udp_dst':
                 # tp_src/dst
                 match_append[key](value, match, attrs)
+            elif key == 'metadata':
+                # metadata
+                metadata = value[0]
+                metadata_mask = value[1]
+                match_append[key](metadata, metadata_mask)
             else:
                 # others
                 match_append[key](value)
@@ -362,12 +394,12 @@ def to_match_ip(value):
     # IP address
     ipv4 = struct.unpack('!I', socket.inet_aton(ip_mask[0]))[0]
     # netmask
-    netmask = ofproto_v1_2_parser.UINT32_MAX
+    netmask = ofproto_v1_3_parser.UINT32_MAX
 
     if len(ip_mask) == 2:
         # Check the mask is CIDR or not.
         if ip_mask[1].isdigit():
-            netmask &= ofproto_v1_2_parser.UINT32_MAX << 32 - int(ip_mask[1])
+            netmask &= ofproto_v1_3_parser.UINT32_MAX << 32 - int(ip_mask[1])
         else:
             netmask = struct.unpack('!I', socket.inet_aton(ip_mask[1]))[0]
 
@@ -390,52 +422,66 @@ def to_match_ipv6(value):
     return ipv6, netmask
 
 
+def to_match_metadata(value):
+    if '/' in value:
+        metadata = value.split('/')
+        return str_to_int(metadata[0]), str_to_int(metadata[1])
+    else:
+        return str_to_int(value), ofproto_v1_3_parser.UINT64_MAX
+
+
 def match_to_str(ofmatch):
-    keys = {ofproto_v1_2.OXM_OF_IN_PORT: 'in_port',
-            ofproto_v1_2.OXM_OF_IN_PHY_PORT: 'in_phy_port',
-            ofproto_v1_2.OXM_OF_ETH_SRC: 'dl_src',
-            ofproto_v1_2.OXM_OF_ETH_DST: 'dl_dst',
-            ofproto_v1_2.OXM_OF_ETH_SRC_W: 'dl_src',
-            ofproto_v1_2.OXM_OF_ETH_DST_W: 'dl_dst',
-            ofproto_v1_2.OXM_OF_ETH_TYPE: 'dl_type',
-            ofproto_v1_2.OXM_OF_VLAN_VID: 'dl_vlan',
-            ofproto_v1_2.OXM_OF_VLAN_PCP: 'vlan_pcp',
-            ofproto_v1_2.OXM_OF_IP_DSCP: 'ip_dscp',
-            ofproto_v1_2.OXM_OF_IP_ECN: 'ip_ecn',
-            ofproto_v1_2.OXM_OF_IPV4_SRC: 'nw_src',
-            ofproto_v1_2.OXM_OF_IPV4_DST: 'nw_dst',
-            ofproto_v1_2.OXM_OF_IPV4_SRC_W: 'nw_src',
-            ofproto_v1_2.OXM_OF_IPV4_DST_W: 'nw_dst',
-            ofproto_v1_2.OXM_OF_IP_PROTO: 'nw_proto',
-            ofproto_v1_2.OXM_OF_TCP_SRC: 'tp_src',
-            ofproto_v1_2.OXM_OF_TCP_DST: 'tp_dst',
-            ofproto_v1_2.OXM_OF_UDP_SRC: 'tp_src',
-            ofproto_v1_2.OXM_OF_UDP_DST: 'tp_dst',
-            ofproto_v1_2.OXM_OF_SCTP_SRC: 'sctp_src',
-            ofproto_v1_2.OXM_OF_SCTP_DST: 'sctp_dst',
-            ofproto_v1_2.OXM_OF_ICMPV4_TYPE: 'icmpv4_type',
-            ofproto_v1_2.OXM_OF_ICMPV4_CODE: 'icmpv4_code',
-            ofproto_v1_2.OXM_OF_MPLS_LABEL: 'mpls_label',
-            ofproto_v1_2.OXM_OF_MPLS_TC: 'mpls_tc',
-            ofproto_v1_2.OXM_OF_ARP_OP: 'arp_op',
-            ofproto_v1_2.OXM_OF_ARP_SPA: 'arp_spa',
-            ofproto_v1_2.OXM_OF_ARP_TPA: 'arp_tpa',
-            ofproto_v1_2.OXM_OF_ARP_SPA_W: 'arp_spa',
-            ofproto_v1_2.OXM_OF_ARP_TPA_W: 'arp_tpa',
-            ofproto_v1_2.OXM_OF_ARP_SHA: 'arp_sha',
-            ofproto_v1_2.OXM_OF_ARP_THA: 'arp_tha',
-            ofproto_v1_2.OXM_OF_ARP_SHA_W: 'arp_sha',
-            ofproto_v1_2.OXM_OF_ARP_THA_W: 'arp_tha',
-            ofproto_v1_2.OXM_OF_IPV6_SRC: 'ipv6_src',
-            ofproto_v1_2.OXM_OF_IPV6_DST: 'ipv6_dst',
-            ofproto_v1_2.OXM_OF_IPV6_SRC_W: 'ipv6_src',
-            ofproto_v1_2.OXM_OF_IPV6_DST_W: 'ipv6_dst',
-            ofproto_v1_2.OXM_OF_IPV6_FLABEL: 'ipv6_flabel',
-            ofproto_v1_2.OXM_OF_ICMPV6_TYPE: 'icmpv6_type',
-            ofproto_v1_2.OXM_OF_ICMPV6_CODE: 'icmpv6_code',
-            ofproto_v1_2.OXM_OF_IPV6_ND_TARGET: 'ipv6_nd_target',
-            ofproto_v1_2.OXM_OF_IPV6_ND_SLL: 'ipv6_nd_sll',
-            ofproto_v1_2.OXM_OF_IPV6_ND_TLL: 'ipv6_nd_tll'}
+    keys = {ofproto_v1_3.OXM_OF_IN_PORT: 'in_port',
+            ofproto_v1_3.OXM_OF_IN_PHY_PORT: 'in_phy_port',
+            ofproto_v1_3.OXM_OF_ETH_SRC: 'dl_src',
+            ofproto_v1_3.OXM_OF_ETH_DST: 'dl_dst',
+            ofproto_v1_3.OXM_OF_ETH_SRC_W: 'dl_src',
+            ofproto_v1_3.OXM_OF_ETH_DST_W: 'dl_dst',
+            ofproto_v1_3.OXM_OF_ETH_TYPE: 'dl_type',
+            ofproto_v1_3.OXM_OF_VLAN_VID: 'dl_vlan',
+            ofproto_v1_3.OXM_OF_VLAN_PCP: 'vlan_pcp',
+            ofproto_v1_3.OXM_OF_IP_DSCP: 'ip_dscp',
+            ofproto_v1_3.OXM_OF_IP_ECN: 'ip_ecn',
+            ofproto_v1_3.OXM_OF_IPV4_SRC: 'nw_src',
+            ofproto_v1_3.OXM_OF_IPV4_DST: 'nw_dst',
+            ofproto_v1_3.OXM_OF_IPV4_SRC_W: 'nw_src',
+            ofproto_v1_3.OXM_OF_IPV4_DST_W: 'nw_dst',
+            ofproto_v1_3.OXM_OF_IP_PROTO: 'nw_proto',
+            ofproto_v1_3.OXM_OF_TCP_SRC: 'tp_src',
+            ofproto_v1_3.OXM_OF_TCP_DST: 'tp_dst',
+            ofproto_v1_3.OXM_OF_UDP_SRC: 'tp_src',
+            ofproto_v1_3.OXM_OF_UDP_DST: 'tp_dst',
+            ofproto_v1_3.OXM_OF_SCTP_SRC: 'sctp_src',
+            ofproto_v1_3.OXM_OF_SCTP_DST: 'sctp_dst',
+            ofproto_v1_3.OXM_OF_ICMPV4_TYPE: 'icmpv4_type',
+            ofproto_v1_3.OXM_OF_ICMPV4_CODE: 'icmpv4_code',
+            ofproto_v1_3.OXM_OF_MPLS_LABEL: 'mpls_label',
+            ofproto_v1_3.OXM_OF_MPLS_TC: 'mpls_tc',
+            ofproto_v1_3.OXM_OF_MPLS_BOS: 'mpls_bos',
+            ofproto_v1_3.OXM_OF_METADATA: 'metadata',
+            ofproto_v1_3.OXM_OF_METADATA_W: 'metadata',
+            ofproto_v1_3.OXM_OF_ARP_OP: 'arp_op',
+            ofproto_v1_3.OXM_OF_ARP_SPA: 'arp_spa',
+            ofproto_v1_3.OXM_OF_ARP_TPA: 'arp_tpa',
+            ofproto_v1_3.OXM_OF_ARP_SPA_W: 'arp_spa',
+            ofproto_v1_3.OXM_OF_ARP_TPA_W: 'arp_tpa',
+            ofproto_v1_3.OXM_OF_ARP_SHA: 'arp_sha',
+            ofproto_v1_3.OXM_OF_ARP_THA: 'arp_tha',
+            ofproto_v1_3.OXM_OF_ARP_SHA_W: 'arp_sha',
+            ofproto_v1_3.OXM_OF_ARP_THA_W: 'arp_tha',
+            ofproto_v1_3.OXM_OF_IPV6_SRC: 'ipv6_src',
+            ofproto_v1_3.OXM_OF_IPV6_DST: 'ipv6_dst',
+            ofproto_v1_3.OXM_OF_IPV6_SRC_W: 'ipv6_src',
+            ofproto_v1_3.OXM_OF_IPV6_DST_W: 'ipv6_dst',
+            ofproto_v1_3.OXM_OF_IPV6_FLABEL: 'ipv6_flabel',
+            ofproto_v1_3.OXM_OF_ICMPV6_TYPE: 'icmpv6_type',
+            ofproto_v1_3.OXM_OF_ICMPV6_CODE: 'icmpv6_code',
+            ofproto_v1_3.OXM_OF_IPV6_ND_TARGET: 'ipv6_nd_target',
+            ofproto_v1_3.OXM_OF_IPV6_ND_SLL: 'ipv6_nd_sll',
+            ofproto_v1_3.OXM_OF_IPV6_ND_TLL: 'ipv6_nd_tll',
+            ofproto_v1_3.OXM_OF_PBB_ISID: 'pbb_isid',
+            ofproto_v1_3.OXM_OF_TUNNEL_ID: 'tunnel_id',
+            ofproto_v1_3.OXM_OF_IPV6_EXTHDR: 'ipv6_exthdr'}
 
     match = {}
     for match_field in ofmatch.fields:
@@ -450,6 +496,9 @@ def match_to_str(ofmatch):
             value = match_ip_to_str(match_field.value, match_field.mask)
         elif key == 'ipv6_src' or key == 'ipv6_dst':
             value = match_ipv6_to_str(match_field.value, match_field.mask)
+        elif key == 'metadata':
+            value = ('%d/%d' % (match_field.value, match_field.mask)
+                     if match_field.mask else '%d' % match_field.value)
         else:
             value = match_field.value
         match.setdefault(key, value)
@@ -525,11 +574,11 @@ def send_stats_request(dp, stats, waiters, msgs):
 
 
 def get_desc_stats(dp, waiters):
-    stats = dp.ofproto_parser.OFPDescStatsRequest(dp)
+    stats = dp.ofproto_parser.OFPDescStatsRequest(dp, 0)
     msgs = []
     send_stats_request(dp, stats, waiters, msgs)
-
     s = {}
+
     for msg in msgs:
         stats = msg.body
         s = {'mfr_desc': stats.mfr_desc,
@@ -565,6 +614,7 @@ def get_queue_stats(dp, waiters):
 
 def get_flow_stats(dp, waiters):
     table_id = dp.ofproto.OFPTT_ALL
+    flags = 0
     out_port = dp.ofproto.OFPP_ANY
     out_group = dp.ofproto.OFPG_ANY
     cookie = 0
@@ -572,7 +622,8 @@ def get_flow_stats(dp, waiters):
     match = dp.ofproto_parser.OFPMatch()
 
     stats = dp.ofproto_parser.OFPFlowStatsRequest(
-        dp, table_id, out_port, out_group, cookie, cookie_mask, match)
+        dp, flags, table_id, out_port, out_group, cookie, cookie_mask,
+        match)
 
     msgs = []
     send_stats_request(dp, stats, waiters, msgs)
@@ -582,6 +633,7 @@ def get_flow_stats(dp, waiters):
         for stats in msg.body:
             actions = actions_to_str(stats.instructions)
             match = match_to_str(stats.match)
+
             s = {'priority': stats.priority,
                  'cookie': stats.cookie,
                  'idle_timeout': stats.idle_timeout,
@@ -601,7 +653,7 @@ def get_flow_stats(dp, waiters):
 
 def get_port_stats(dp, waiters):
     stats = dp.ofproto_parser.OFPPortStatsRequest(
-        dp, dp.ofproto.OFPP_ANY, 0)
+        dp, 0, dp.ofproto.OFPP_ANY)
     msgs = []
     send_stats_request(dp, stats, waiters, msgs)
 
@@ -626,25 +678,118 @@ def get_port_stats(dp, waiters):
     return ports
 
 
+def get_meter_stats(dp, waiters):
+    stats = dp.ofproto_parser.OFPMeterStatsRequest(
+        dp, 0, dp.ofproto.OFPM_ALL)
+    msgs = []
+    send_stats_request(dp, stats, waiters, msgs)
+
+    meters = []
+    for msg in msgs:
+        for stats in msg.body:
+            bands = []
+            for band in stats.band_stats:
+                b = {'packet_band_count': band.packet_band_count,
+                     'byte_band_count': band.byte_band_count}
+                bands.append(b)
+            s = {'meter_id': stats.meter_id,
+                 'len': stats.len,
+                 'flow_count': stats.flow_count,
+                 'packet_in_count': stats.packet_in_count,
+                 'byte_in_count': stats.byte_in_count,
+                 'band_stats': bands}
+            meters.append(s)
+    meters = {str(dp.id): meters}
+    return meters
+
+
+def get_meter_features(dp, waiters):
+
+    ofp = dp.ofproto
+    type_convert = {ofp.OFPMBT_DROP: 'DROP',
+                    ofp.OFPMBT_DSCP_REMARK: 'DSCP_REMARK'}
+
+    stats = dp.ofproto_parser.OFPMeterFeaturesStatsRequest(dp, 0)
+    msgs = []
+    send_stats_request(dp, stats, waiters, msgs)
+
+    features = []
+    for msg in msgs:
+        for feature in msg.body:
+            band_types = []
+            for k, v in type_convert.items():
+                if (1 << k) & feature.band_types:
+                    band_types.append(v)
+            f = {'max_meter': feature.max_meter,
+                 'band_types': band_types,
+                 'max_band': feature.max_band,
+                 'max_color': feature.max_color}
+            features.append(f)
+    features = {str(dp.id): features}
+    return features
+
+
+def get_meter_config(dp, waiters):
+    flags = {dp.ofproto.OFPMF_KBPS: 'KBPS',
+             dp.ofproto.OFPMF_PKTPS: 'PKTPS',
+             dp.ofproto.OFPMF_BURST: 'BURST',
+             dp.ofproto.OFPMF_STATS: 'STATS'}
+
+    band_type = {dp.ofproto.OFPMBT_DROP: 'DROP',
+                 dp.ofproto.OFPMBT_DSCP_REMARK: 'REMARK',
+                 dp.ofproto.OFPMBT_EXPERIMENTER: 'EXPERIMENTER'}
+
+    stats = dp.ofproto_parser.OFPMeterConfigStatsRequest(
+        dp, 0, dp.ofproto.OFPM_ALL)
+    msgs = []
+    send_stats_request(dp, stats, waiters, msgs)
+
+    configs = []
+    for msg in msgs:
+        for config in msg.body:
+            bands = []
+            for band in config.bands:
+                b = {'type': band_type.get(band.type, ''),
+                     'rate': band.rate,
+                     'burst_size': band.burst_size}
+                if band.type == dp.ofproto.OFPMBT_DSCP_REMARK:
+                    b['prec_level'] = band.prec_level
+                elif band.type == dp.ofproto.OFPMBT_EXPERIMENTER:
+                    b['experimenter'] = band.experimenter
+                bands.append(b)
+            c_flags = []
+            for k, v in flags.items():
+                if k & config.flags:
+                    c_flags.append(v)
+            c = {'flags': c_flags,
+                 'meter_id': config.meter_id,
+                 'bands': bands}
+            configs.append(c)
+    configs = {str(dp.id): configs}
+    return configs
+
+
 def get_group_stats(dp, waiters):
     stats = dp.ofproto_parser.OFPGroupStatsRequest(
-        dp, dp.ofproto.OFPG_ALL, 0)
+        dp, 0, dp.ofproto.OFPG_ALL)
     msgs = []
     send_stats_request(dp, stats, waiters, msgs)
 
     groups = []
     for msg in msgs:
         for stats in msg.body:
-            bucket_counters = []
-            for bucket_counter in stats.bucket_counters:
-                c = {'packet_count': bucket_counter.packet_count,
-                     'byte_count': bucket_counter.byte_count}
-                bucket_counters.append(c)
+            bucket_stats = []
+            for bucket_stat in stats.bucket_stats:
+                c = {'packet_count': bucket_stat.packet_count,
+                     'byte_count': bucket_stat.byte_count}
+                bucket_stats.append(c)
             g = {'group_id': stats.group_id,
                  'ref_count': stats.ref_count,
                  'packet_count': stats.packet_count,
                  'byte_count': stats.byte_count,
-                 'bucket_stats': bucket_counters}
+                 'duration_sec': stats.duration_sec,
+                 'duration_nsec': stats.duration_nsec,
+                 'bucket_stats': bucket_stats}
             groups.append(g)
     groups = {str(dp.id): groups}
     return groups
@@ -674,7 +819,9 @@ def get_group_features(dp, waiters):
                    ofp.OFPAT_GROUP: 'GROUP',
                    ofp.OFPAT_SET_NW_TTL: 'SET_NW_TTL',
                    ofp.OFPAT_DEC_NW_TTL: 'DEC_NW_TTL',
-                   ofp.OFPAT_SET_FIELD: 'SET_FIELD'}
+                   ofp.OFPAT_SET_FIELD: 'SET_FIELD',
+                   ofp.OFPAT_PUSH_PBB: 'PUSH_PBB',
+                   ofp.OFPAT_POP_PBB: 'POP_PBB'}
 
     stats = dp.ofproto_parser.OFPGroupFeaturesStatsRequest(dp, 0)
     msgs = []
@@ -764,6 +911,51 @@ def mod_flow_entry(dp, flow, cmd):
     dp.send_msg(flow_mod)
 
 
+def mod_meter_entry(dp, flow, cmd):
+
+    flags_convert = {'KBPS': dp.ofproto.OFPMF_KBPS,
+                     'PKTPS': dp.ofproto.OFPMF_PKTPS,
+                     'BURST': dp.ofproto.OFPMF_BURST,
+                     'STATS': dp.ofproto.OFPMF_STATS}
+
+    flow_flags = flow.get('flags')
+    if not isinstance(flow_flags, list):
+        flow_flags = [flow_flags]
+    flags = 0
+    for flag in flow_flags:
+        flags |= flags_convert.get(flag, 0)
+    if not flags:
+        LOG.debug('Unknown flags: %s', flow.get('flags'))
+
+    meter_id = int(flow.get('meter_id', 0))
+
+    bands = []
+    for band in flow.get('bands', []):
+        band_type = band.get('type')
+        rate = int(band.get('rate', 0))
+        burst_size = int(band.get('burst_size', 0))
+        if band_type == 'DROP':
+            bands.append(
+                dp.ofproto_parser.OFPMeterBandDrop(rate, burst_size))
+        elif band_type == 'REMARK':
+            prec_level = int(band.get('prec_level', 0))
+            bands.append(
+                dp.ofproto_parser.OFPMeterBandDscpRemark(
+                    rate, burst_size, prec_level))
+        elif band_type == 'EXPERIMENTER':
+            experimenter = int(band.get('experimenter', 0))
+            bands.append(
+                dp.ofproto_parser.OFPMeterBandExperimenter(
+                    rate, burst_size, experimenter))
+        else:
+            LOG.debug('Unknown band type: %s', band_type)
+
+    meter_mod = dp.ofproto_parser.OFPMeterMod(
+        dp, cmd, flags, meter_id, bands)
+
+    dp.send_msg(meter_mod)
+
+
 def mod_group_entry(dp, group, cmd):
 
     type_convert = {'ALL': dp.ofproto.OFPGT_ALL,
