@@ -335,6 +335,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def ip_address(self):
         return self._neigh_conf.ip_address
 
+    @property
+    def protocol(self):
+        return self._protocol
+
     @property
     def host_bind_ip(self):
         return self._host_bind_ip
@@ -586,7 +590,10 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         point/local ip address.
         """
         # By default we use BGPS's interface IP with this peer as next_hop.
-        next_hop = self.host_bind_ip
+        if self._neigh_conf.next_hop:
+            next_hop = self._neigh_conf.next_hop
+        else:
+            next_hop = self.host_bind_ip
         if route_family == RF_IPv6_VPN:
             # Next hop ipv4_mapped ipv6
             def _ipv4_mapped_ipv6(ipv4):
@@ -773,12 +780,12 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             if unkown_opttrans_attrs:
                 new_pathattr.extend(unkown_opttrans_attrs.values())
 
-            if isinstance(path, Ipv4Path):
-                update = BGPUpdate(path_attributes=new_pathattr,
-                                   nlri=nlri_list)
-            else:
-                update = BGPUpdate(path_attributes=new_pathattr)
-            return update
+        if isinstance(path, Ipv4Path):
+            update = BGPUpdate(path_attributes=new_pathattr,
+                               nlri=nlri_list)
+        else:
+            update = BGPUpdate(path_attributes=new_pathattr)
+        return update
 
     def _connect_loop(self, client_factory):
         """In the current greeenlet we try to establish connection with peer.
@@ -815,10 +822,12 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 LOG.debug('%s trying to connect to %s' % (self, peer_address))
                 tcp_conn_timeout = self._common_conf.tcp_conn_timeout
                 try:
+                    password = self._neigh_conf.password
                     sock = self._connect_tcp(peer_address,
                                              client_factory,
                                              time_out=tcp_conn_timeout,
-                                             bind_address=bind_addr)
+                                             bind_address=bind_addr,
+                                             password=password)
                 except socket.error:
                     self.state.bgp_state = const.BGP_FSM_ACTIVE
                     LOG.debug('Socket could not be created in time (%s secs),'
@@ -835,10 +844,8 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         self._protocol = proto
 
         # Update state attributes
-        self.state.peer_ip, self.state.peer_port = \
-            self._protocol.get_peername()
-        self.state.local_ip, self.state.local_port = \
-            self._protocol.get_sockname()
+        self.state.peer_ip, self.state.peer_port = self._protocol._remotename
+        self.state.local_ip, self.state.local_port = self._protocol._localname
 #         self.state.bgp_state = self._protocol.state
         # Stop connect_loop retry timer as we are now connected
         if self._protocol and self._connect_retry_event.is_set():
@@ -1231,7 +1238,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                       ' UPDATE. %s' % update_msg)
             return
 
-        if msg_rf in (RF_IPv4_VPN, RF_IPv6_UC):
+        if msg_rf in (RF_IPv4_VPN, RF_IPv6_VPN):
             # Check if we have Extended Communities Attribute.
             # TODO(PH): Check if RT_NLRI afi/safi will ever have this attribute
             ext_comm_attr = umsg_pattrs.get(BGP_ATTR_TYPE_EXTENDED_COMMUNITIES)
