@@ -18,6 +18,7 @@
 """
 from abc import abstractmethod
 import logging
+import netaddr
 
 from ryu.lib.packet.bgp import RF_IPv4_UC
 from ryu.lib.packet.bgp import RF_IPv6_UC
@@ -37,6 +38,7 @@ from ryu.services.protocols.bgp.rtconf.base import BaseConf
 from ryu.services.protocols.bgp.rtconf.base import BaseConfListener
 from ryu.services.protocols.bgp.rtconf.base import CAP_ENHANCED_REFRESH
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV4
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV6
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV4
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV6
 from ryu.services.protocols.bgp.rtconf.base import CAP_REFRESH
@@ -69,12 +71,15 @@ ENABLED = 'enabled'
 CHANGES = 'changes'
 LOCAL_ADDRESS = 'local_address'
 LOCAL_PORT = 'local_port'
+PEER_NEXT_HOP = 'next_hop'
+PASSWORD = 'password'
 
 # Default value constants.
 DEFAULT_CAP_GR_NULL = True
 DEFAULT_CAP_REFRESH = True
 DEFAULT_CAP_ENHANCED_REFRESH = False
 DEFAULT_CAP_MBGP_IPV4 = True
+DEFAULT_CAP_MBGP_IPV6 = False
 DEFAULT_CAP_MBGP_VPNV4 = False
 DEFAULT_CAP_MBGP_VPNV6 = False
 DEFAULT_HOLD_TIME = 40
@@ -107,20 +112,39 @@ def validate_changes(changes):
     return changes
 
 
+def valid_ip_address(addr):
+    if not netaddr.valid_ipv4(addr) and not netaddr.valid_ipv6(addr):
+        return False
+    return True
+
+
 @validate(name=IP_ADDRESS)
 def validate_ip_address(ip_address):
-    if not is_valid_ipv4(ip_address):
+    if not valid_ip_address(ip_address):
         raise ConfigValueError(desc='Invalid neighbor ip_address: %s' %
                                ip_address)
-    return ip_address
+    return str(netaddr.IPAddress(ip_address))
 
 
 @validate(name=LOCAL_ADDRESS)
 def validate_local_address(ip_address):
-    if not is_valid_ipv4(ip_address):
+    if not valid_ip_address(ip_address):
         raise ConfigValueError(desc='Invalid local ip_address: %s' %
                                ip_address)
-    return ip_address
+    return str(netaddr.IPAddress(ip_address))
+
+
+@validate(name=PEER_NEXT_HOP)
+def validate_next_hop(ip_address):
+    if not valid_ip_address(ip_address):
+        raise ConfigValueError(desc='Invalid next_hop ip_address: %s' %
+                               ip_address)
+    return str(netaddr.IPAddress(ip_address))
+
+
+@validate(name=PASSWORD)
+def validate_password(password):
+    return password
 
 
 @validate(name=LOCAL_PORT)
@@ -149,12 +173,14 @@ class NeighborConf(ConfWithId, ConfWithStats):
     VALID_EVT = frozenset([UPDATE_ENABLED_EVT, UPDATE_MED_EVT])
     REQUIRED_SETTINGS = frozenset([REMOTE_AS, IP_ADDRESS])
     OPTIONAL_SETTINGS = frozenset([CAP_REFRESH,
-                                   CAP_ENHANCED_REFRESH, CAP_MBGP_VPNV4,
-                                   CAP_MBGP_IPV4, CAP_MBGP_VPNV6,
+                                   CAP_ENHANCED_REFRESH,
+                                   CAP_MBGP_IPV4, CAP_MBGP_IPV6,
+                                   CAP_MBGP_VPNV4, CAP_MBGP_VPNV6,
                                    CAP_RTC, RTC_AS, HOLD_TIME,
                                    ENABLED, MULTI_EXIT_DISC, MAX_PREFIXES,
                                    ADVERTISE_PEER_AS, SITE_OF_ORIGINS,
-                                   LOCAL_ADDRESS, LOCAL_PORT])
+                                   LOCAL_ADDRESS, LOCAL_PORT,
+                                   PEER_NEXT_HOP, PASSWORD])
 
     def __init__(self, **kwargs):
         super(NeighborConf, self).__init__(**kwargs)
@@ -166,6 +192,8 @@ class NeighborConf(ConfWithId, ConfWithStats):
             CAP_ENHANCED_REFRESH, DEFAULT_CAP_ENHANCED_REFRESH, **kwargs)
         self._settings[CAP_MBGP_IPV4] = compute_optional_conf(
             CAP_MBGP_IPV4, DEFAULT_CAP_MBGP_IPV4, **kwargs)
+        self._settings[CAP_MBGP_IPV6] = compute_optional_conf(
+            CAP_MBGP_IPV6, DEFAULT_CAP_MBGP_IPV6, **kwargs)
         self._settings[CAP_MBGP_VPNV4] = compute_optional_conf(
             CAP_MBGP_VPNV4, DEFAULT_CAP_MBGP_VPNV4, **kwargs)
         self._settings[CAP_MBGP_VPNV6] = compute_optional_conf(
@@ -201,6 +229,12 @@ class NeighborConf(ConfWithId, ConfWithStats):
         self._settings[LOCAL_PORT] = compute_optional_conf(
             LOCAL_PORT, None, **kwargs)
 
+        self._settings[PEER_NEXT_HOP] = compute_optional_conf(
+            PEER_NEXT_HOP, None, **kwargs)
+
+        self._settings[PASSWORD] = compute_optional_conf(
+            PASSWORD, None, **kwargs)
+
         # RTC configurations.
         self._settings[CAP_RTC] = \
             compute_optional_conf(CAP_RTC, DEFAULT_CAP_RTC, **kwargs)
@@ -253,6 +287,14 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def host_bind_port(self):
         return self._settings[LOCAL_PORT]
 
+    @property
+    def next_hop(self):
+        return self._settings[PEER_NEXT_HOP]
+
+    @property
+    def password(self):
+        return self._settings[PASSWORD]
+
     # =========================================================================
     # Optional attributes with valid defaults.
     # =========================================================================
@@ -273,6 +315,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def cap_mbgp_ipv4(self):
         return self._settings[CAP_MBGP_IPV4]
 
+    @property
+    def cap_mbgp_ipv6(self):
+        return self._settings[CAP_MBGP_IPV6]
+
     @property
     def cap_mbgp_vpnv4(self):
         return self._settings[CAP_MBGP_VPNV4]
@@ -347,6 +393,11 @@ class NeighborConf(ConfWithId, ConfWithStats):
                 BGPOptParamCapabilityMultiprotocol(
                     RF_IPv4_UC.afi, RF_IPv4_UC.safi))
 
+        if self.cap_mbgp_ipv6:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_IPv6_UC.afi, RF_IPv6_UC.safi))
+
         if self.cap_mbgp_vpnv4:
             mbgp_caps.append(
                 BGPOptParamCapabilityMultiprotocol(
