@@ -20,6 +20,8 @@ import logging
 import socket
 import struct
 import traceback
+from socket import IPPROTO_TCP, TCP_NODELAY
+from eventlet import semaphore
 
 from ryu.lib.packet import bgp
 from ryu.lib.packet.bgp import RouteFamily
@@ -94,14 +96,18 @@ class BgpProtocol(Protocol, Activity):
         # Validate input.
         if socket is None:
             raise ValueError('Invalid arguments passed.')
-        activity_name = ('BgpProtocol %s, %s, %s' % (
-            is_reactive_conn, socket.getpeername(), socket.getsockname())
-        )
+        self._remotename = self.get_remotename(socket)
+        self._localname = self.get_localname(socket)
+        activity_name = ('BgpProtocol %s, %s, %s' % (is_reactive_conn,
+                                                     self._remotename,
+                                                     self._localname))
         Activity.__init__(self, name=activity_name)
         # Intialize instance variables.
         self._peer = None
         self._recv_buff = ''
         self._socket = socket
+        self._socket.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
+        self._sendlock = semaphore.Semaphore()
         self._signal_bus = signal_bus
         self._holdtime = None
         self._keepalive = None
@@ -118,12 +124,6 @@ class BgpProtocol(Protocol, Activity):
         self.recv_open_msg = None
         self._is_bound = False
 
-    def get_peername(self):
-        return self._socket.getpeername()
-
-    def get_sockname(self):
-        return self._socket.getsockname()
-
     @property
     def is_reactive(self):
         return self._is_reactive
@@ -142,8 +142,8 @@ class BgpProtocol(Protocol, Activity):
                              '`BgpProtocol`')
 
         # Compare protocol connection end point's addresses
-        if (self.get_peername()[0] == other_protocol.get_peername()[0] and
-                self.get_sockname()[0] == other_protocol.get_sockname()[0]):
+        if (self._remotename[0] == other_protoco._remotename[0] and
+                self._localname[0] == other_protocol._localname[0]):
             return True
 
         return False
@@ -159,8 +159,8 @@ class BgpProtocol(Protocol, Activity):
         if not self.state == BGP_FSM_OPEN_CONFIRM:
             raise BgpProtocolException(desc='Can access remote router id only'
                                             ' after open message is received')
-        remote_id = self.recv_open_msg.bgpid
-        local_id = self.sent_open_msg.bgpid
+        remote_id = self.recv_open_msg.bgp_identifier
+        local_id = self.sent_open_msg.bgp_identifier
         return from_inet_ptoi(local_id) > from_inet_ptoi(remote_id)
 
     def is_enhanced_rr_cap_valid(self):
@@ -360,31 +360,35 @@ class BgpProtocol(Protocol, Activity):
         """
         notification = BGPNotification(code, subcode)
         reason = notification.reason
-        self._socket.sendall(notification.serialize())
+        self._send_with_lock(notification)
         self._signal_bus.bgp_error(self._peer, code, subcode, reason)
-        LOG.error(
-            'Sent notification to %r >> %s' %
-            (self._socket.getpeername(), notification)
-        )
+        LOG.error('Sent notification to %r >> %s' % (self._localname(),
+                                                     notification))
         self._socket.close()
 
+    def _send_with_lock(self, msg):
+        self._sendlock.acquire()
+        try:
+            self._socket.sendall(msg.serialize())
+        except socket.error as err:
+            self.connection_lost('failed to write to socket')
+        finally:
+            self._sendlock.release()
+
     def send(self, msg):
         if not self.started:
             raise BgpProtocolException('Tried to send message to peer when '
                                        'this protocol instance is not started'
                                        ' or is no longer is started state.')
-        # get peername before senging msg because sending msg can occur
-        # conncetion lost
-        peername = self.get_peername()
-        self._socket.sendall(msg.serialize())
+        self._send_with_lock(msg)
 
         if msg.type == BGP_MSG_NOTIFICATION:
             LOG.error('Sent notification to %s >> %s' %
-                      (peername, msg))
+                      (self._remotename, msg))
 
             self._signal_bus.bgp_notification_sent(self._peer, msg)
         else:
-            LOG.debug('Sent msg to %s >> %s' % (peername, msg))
+            LOG.debug('Sent msg to %s >> %s' % (self._remotename, msg))
 
     def stop(self):
         Activity.stop(self)
@@ -430,8 +434,7 @@ class BgpProtocol(Protocol, Activity):
         message except for *Open* and *Notification* message. On receiving
         *Notification* message we close connection with peer.
         """
-        LOG.debug('Received msg from %s << %s' % (str(self.get_peername()),
-                                                  msg))
+        LOG.debug('Received msg from %s << %s' % (self._remotename, msg))
 
         # If we receive open message we try to bind to protocol
         if (msg.type == BGP_MSG_OPEN):
